#!/usr/bin/perl -w

use strict;
use DBI;
use Data::Dumper;
use Getopt::Std;
use MIME::Lite;
use Sys::Hostname;
use FindBin qw($Bin);

my $ME_ADDRESS="edwin\@FreeBSD.org";
my $ME_NAME="Edwin Groothuis";

my $DBDIR="";
{
    my @a=split("/",$Bin);
    pop(@a);
    $DBDIR=join("/",@a)."/db";
}

my %opts;
getopts("m:p:s",\%opts);

my $MAINTAINER="";
$MAINTAINER=$opts{m} if (defined $opts{m});
my $PORT="";
$PORT=$opts{p} if (defined $opts{p});
my $EMAIL=0;
$EMAIL=1 if (defined $opts{s});

my $dbh=DBI->connect("dbi:SQLite:dbname=$DBDIR/newports.db","","");

my @maintainers=();
{
    my $portonly="";
    $portonly="AND directory='$PORT'" if ($PORT);
    my $sth=$dbh->prepare("
	SELECT	distinct maintainer
	FROM	newports
	WHERE	reported=0
		$portonly
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	push(@maintainers,$a[0]);
    }
}

foreach my $maintainer (sort(@maintainers)) {
    next if ($MAINTAINER && lc($MAINTAINER) ne lc($maintainer));

    my %ports=();
    my %urllist=();
    my %directorylist=();

    my $portonly="";
    $portonly="AND directory='$PORT'" if ($PORT);
    my $sth=$dbh->prepare("
	SELECT	distinct category,port
	FROM	newports
	WHERE	maintainer like ?
	AND	reported=0
		$portonly
    ");
    $sth->execute($maintainer);
    while (my @a=$sth->fetchrow_array()) {
	$ports{$a[0]}{$a[1]}=1;
    }

    my %urls=();
    foreach my $category (sort(keys(%ports))) {
	foreach my $port (sort(keys(%{$ports{$category}}))) {

	    my $sth=$dbh->prepare("
		SELECT	distinct directory,currentversion,newversion,url
		FROM	newports
		WHERE	maintainer=?
		AND	port=?
		AND	category=?
		AND	reported=0
		ORDER BY newversion,url
	    ");
	    $sth->execute($maintainer,$port,$category);
	    while (my @a=$sth->fetchrow_array()) {
		$urls{$a[0]}{$a[1]}{$a[2]}{$a[3]}=0;
		$directorylist{$a[0]}{port}=$port;
		$directorylist{$a[0]}{category}=$category;
	    }
	}
    }

    my $txt="";
    $txt="
Dear $maintainer,

As maintainer of one or more FreeBSD ports, I would like to inform
you that a new version is available for the following port(s):

Port                                           Current version New version URLs
---------------------------------------------- --------------- ----------- ----
";

    my $urls=0;
    foreach my $directory (sort(keys(%urls))) {
	foreach my $curversion (sort(keys(%{$urls{$directory}}))) {
	    foreach my $newversion (sort(keys(%{$urls{$directory}{$curversion}}))) {
		$txt.=sprintf("%-46s %-15s %-11s [%d]\n",$directory,$curversion,$newversion,++$urls);
		foreach my $url (sort(keys(%{$urls{$directory}{$curversion}{$newversion}}))) {
		    $urllist{$urls}{$url}=1;
		}
	    }
	}
    }

    $txt.="
Please keep in mind that...

- the check was only done in relation to the current version, so
  it is possible that the new version is not the newest version
  available on the master sites.

- the full check of all ports takes more than two days, so it is
  possible that a new version is already commited. In that case,
  please ignore this email.

- this is the only time you will get an alert about the above
  mentioned port versions on the below mentioned URLs. When the
  port version is updated, a check will again be done for it. When
  a new URL with a newer version is found, a check will be done for
  it again.

- Not interested in further alerts? Send me the email address you
  are received it on for a full block, or the name of the port for
  a port-only block, or the name of a host for a host-only block.

- An overview of all newly discovered distfiles can be found at
  http://edwin.adsl.barnet.com.au/~edwin/ports/. The page keeps
  unchanged until the next run.

URLS
";

    foreach my $c (sort numeric_sort (keys(%urllist))) {
	my $i=0;
	foreach my $url (sort(keys(%{$urllist{$c}}))) {
	    if ($i==0) {
		$txt.=sprintf("[%d]%s ",$c,$c<10?"  ":$c<100?" ":"");
	    } else {
		$txt.="      ";
	    }
	    $txt.=sprintf("%s\n",$url);
	    $i++;
	}
    }

    if ($EMAIL) {
	print "Sending email to $maintainer\n";
	if ($ME ne $maintainer && hostname ne "k7.mavetju") {
	    print "Mail not sent out: you are not the maintainer.\n";
	} else {
	    my $msg=MIME::Lite->new(
		From	=> "$ME_NAME <$ME_ADDRESS>",
		To	=> $maintainer,
		Subject	=> "[FreeBSD Ports Version Check] Newer available versions for ports you maintain",
		Data	=> $txt
	    );
	    $msg->send();
	}
    } else {
	print $txt;
    }
}


sub numeric_sort {
     return $a <=> $b;
}
